package de.fraunhofer.iem.dataprovider.taskManager.tasks.dataQuery

import de.fraunhofer.iem.dataprovider.configuration.OpenCodeGitlabApiProperties
import de.fraunhofer.iem.dataprovider.repository.dto.RepositoryDetailsCreateDto
import de.fraunhofer.iem.dataprovider.repository.entity.RepositoryEntity
import de.fraunhofer.iem.dataprovider.repository.service.RepositoryService
import de.fraunhofer.iem.dataprovider.taskManager.events.Event
import de.fraunhofer.iem.dataprovider.taskManager.events.GetRepositoryDetailsDoneEvent
import de.fraunhofer.iem.dataprovider.taskManager.tasks.Task
import org.gitlab4j.api.GitLabApi
import org.gitlab4j.api.models.Commit
import org.gitlab4j.api.models.Project
import java.util.*

class GetRepositoryDetailsTask(
    private val repoId: UUID,
    gitlabConfiguration: OpenCodeGitlabApiProperties,
    private val repositoryService: RepositoryService,
    override val responseChannel: suspend (event: Event) -> Unit,
) : Task() {
    private val gitlabApi: GitLabApi = GitLabApi(gitlabConfiguration.host, gitlabConfiguration.accessToken)

    override suspend fun execute() {
        val repositoryEntity = repositoryService.findRepoByID(repoId)
        logger.info(repositoryEntity.toString())

        if (repositoryEntity != null) {
            val project = gitlabApi.projectApi.getProject(repositoryEntity.repoId)
            // Note: We only take commits from the default branch
            val commits = gitlabApi.commitsApi.getCommits(project.id, project.defaultBranch, ".")

            val numberOfCommits = commits.count()
            val numberOfSignedCommits = getNumberOfSignedCommits(repositoryEntity, commits)
            val isDefaultBranchProtected = isDefaultBranchProtected(repositoryEntity, project)
            val repositoryDetailsCreateDto = RepositoryDetailsCreateDto(
                repositoryEntity,
                numberOfCommits,
                numberOfSignedCommits,
                isDefaultBranchProtected
            )

            logger.info("Collected repository details from $repoId (${repositoryDetailsCreateDto}) successfully")
            val repositoryDetailsEntity = repositoryService.createRepositoryDetails(repositoryDetailsCreateDto)
            logger.info("Stored repository details for $repoId (${repositoryDetailsEntity}) successfully")

            responseChannel(GetRepositoryDetailsDoneEvent(taskID, repoId, repositoryDetailsEntity))
        } else {
            logger.error("Repository $repoId can not be found in the database")
        }
    }

    // TODO: This should probably live somewhere else and encapsulate the logic
    private fun getNumberOfSignedCommits(repositoryEntity: RepositoryEntity, commits: List<Commit>): Int {
        var numberOfSignedCommits = 0
        for (commit in commits) {
            val signature = gitlabApi.commitsApi.getOptionalGpgSignature(repositoryEntity.repoId, commit.id)
            if (signature !== null) {
                numberOfSignedCommits++
            }
        }
        return numberOfSignedCommits
    }

    // TODO: This should probably live somewhere else and encapsulate the logic
    private fun isDefaultBranchProtected(repositoryEntity: RepositoryEntity, project: Project): Boolean {
        return try {
            val defaultBranchName = project.defaultBranch
            val branch = gitlabApi.repositoryApi.getBranch(repositoryEntity.repoId, defaultBranchName)
            branch.protected
        } catch (e: Exception) {
            // in theory, error probably happens if branch can't be found. In this case we default to false
            false
        }
    }
}
